#!/usr/local/bin/perl
###############################################################################
#                 MRAT - Multi Router Automated Telnet version 0.61           #
#  Origional "client" code from the Perl man pages. Many thanks to whoever    #
#  wrote that.                                                                #
#                                                                             #
#  Copyright (C) 2000  Joe Watkins (nerull@nothingnet.com)                    #
#   This program is free software; you can redistribute it and/or             #
#   modify it under the terms of the GNU General Public License               #
#   as published by the Free Software Foundation; either version 2            #
#   of the License, or (at your option) any later version.                    #
#                                                                             #
#   This program is distributed in the hope that it will be useful,           #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of            #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
#   GNU General Public License for more details.                              #
#                                                                             #
#   You should have received a copy of the GNU General Public License         #
#   along with this program; if not, write to the Free Software               #
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA02111-1307,USA.  #
#                                                                             #
###############################################################################
    use Getopt::Std; # So we can do the processing of the command line options
    use IO::Socket;  # For the connection

 	 # Telnet port.

	$port = '23';

	#Check the command line options

die "Usage: $0 -r <routersfile> -c <commandfile> -o <outputlog>\n" if (@ARGV < 2);

if (getopts('r:c:o:')){ # Only options r, o and c are valid
}else{
       exit; # If not r, o or c exit
     }

####################
# The main program #
####################
read_commands();
read_routers();
openoutputfile();

foreach $routerline (@routers)      # assign @routers to $routerline, line one at a time
{    
  parse_router_file();
  connect_and_do();
}
close(LOG); # Close log file
exit;

################
# sub functions#
################

sub connect_and_do()
{
    # create a tcp connection to the specified host and port
    $handle = IO::Socket::INET->new(Proto     => "tcp",
                                    PeerAddr  => $routerarray[0],
                                    PeerPort  => $port)
           or die "can't connect to port $port on $routerarray[0]: $!";


    $handle->autoflush(1);              # so output gets there right away

    # split the program into two processes, identical twins
    die "can't fork: $!" unless defined($childpid = fork());


    # the if{} block runs only in the parent process
    if ($childpid) {
        # copy the socket to standard output
        while (defined ($line = <$handle>)) {
            print STDOUT $line;
	    print LOG $line; # Also write to log file
	}
    }
    # the else{} block runs only in the child process
    else {
		login();
		print_commands();
		exit;
	  }
wait;
return 0;
}

sub print_commands()
{
foreach $commandline (@commands){    # For each line of the commands file
$commandline =~ s/!var1/$uservar1/g; # replace !varx with the user variable
$commandline =~ s/!var2/$uservar2/g;
$commandline =~ s/!var3/$uservar3/g;
$commandline =~ s/!var4/$uservar4/g;
print $handle $commandline;
                            }
}

sub login()
{
print $handle $username."\n" if $username; # Use a username only if there is one
print $handle $routerarray[1]."\n";
print $handle "enable\n";
print $handle $routerarray[2]."\n";
}

sub read_commands()
{
if (open(COMMANDS, $opt_c)){     # Open commands file
@commands = <COMMANDS>;          # Read the commands file into an array
close(COMMANDS);
                        }else{
				print "Can't open $opt_c\n";
                                exit;
			     }
}

sub read_routers()
{
if (open(ROUTERS, $opt_r)){               # Open routers file
@routers = <ROUTERS> ;                     # Read routers file an array
close(ROUTERS) ;
                        }else{
				print "Can't open $opt_f\n";
				exit;
			     }
}

sub parse_router_file(){
@routerarray = split(/:/, $routerline); #Read current line from router
                                        # file into an array, split by :

$uservar1 = $routerarray[4];   # Set the users variable from the 4th field
$uservar2 = $routerarray[5];   # to the 7th field of the routers file
$uservar3 = $routerarray[6];
$uservar4 = $routerarray[7];
$username = $routerarray[3]; # Get the username
}

sub openoutputfile(){
# Open log file for append. If user used -o open that file otherwise use default
# The idea for letting the user choose an output file and some of the code
# to do this is from Ted Serreyn (ted@serreyn.com)
if ($opt_o) {
open(LOG, '>>'.$opt_o) || die "could not open output file";
}
else {
  open(LOG, '>>output.log');     # Open log file for append
}
}
